"""
Configs migration

Generation datetime: 2022-10-11 08:51:52.226473
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "acbae50c"
DOWN_REVISION = "eecb824f"


limitationsToModify: List[LimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="BACKPORT3_ENABLE_WS_EVENTS",
        validationSchema={"type": "boolean"},
        defaultValue=False,
        description="Whether to enable ws events support",
        services=["luna-backport3"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="BACKPORT3_EVENTS_DB_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "password": {"type": "string"},
                "channel": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels"],
                },
            },
            "additionalProperties": False,
        },
        defaultValue={
            "host": "127.0.0.1",
            "port": 6379,
            "password": "",
            "channel": "luna-backport3",
            "sentinel": {"master_name": "luna_backport3_master", "sentinels": []},
        },
        description="Redis database address",
        services=["luna-backport3"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="BACKPORT3_ENABLE_WS_EVENTS")
    mh.removeLimitation(limitationName="BACKPORT3_EVENTS_DB_ADDRESS")
