"""
Configs migration example

Generation datetime: 2020-11-17 14:37:27.409507
"""

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify
from sqlalchemy import and_, delete, select

from ..models import Setting, Tag

# DO NOT CHANGE MANUALLY AUTO-GENERATED REVISIONS
REVISION = "random"
DOWN_REVISION = "base_revision"


def magicFunction():
    """
    Developer can create any function such as this if there no required functionality in 'mh' methods or
    LimitationToModify methods.

    Use `mh.connection` as connection to database.
    """
    filters = and_(Tag.id == Setting.id, Tag.name == "bad tag")
    selectStatement = select([Setting.name]).where(filters)
    selectResults = mh.connection.execute(selectStatement).cursor.fetchall()
    print(f"Hello, next settings have bad tag: {selectResults}")
    deleteStatement = delete(Setting).where(filters)
    deleteRes = mh.connection.execute(deleteStatement).cursor.fetchall()
    if deleteRes.rowcount == len(selectResults):
        print("There is no more settings with bad tags! =)")
    else:
        print("Something goes wrong, need help =(")


"""
To update setting values with non-default value, use settingUpgradeFunction and settingDowngradeFunction
Each of settingUpgradeFunction and settingDowngradeFunction must receive old setting value and return the new one
If exists old/newDefaultValue and settingUpdate/DowngradeFunction, function executing priority is higher 
"""
limitationsToModify = [
    LimitationToModify(
        limitationName="LUNA_FACES_DB_PING_MAX_COUNT",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform", "luna-queue"],
    ),
    LimitationToModify(limitationName="DATABASE_NUMBER"),
    LimitationToModify(
        limitationName="STORAGE_TYPE",
        oldValidationSchema={"type": "string", "enum": ["LOCAL", "S3"]},
        newValidationSchema={"type": "string", "enum": ["LOCAL", "S3", "S4"]},
    ),
    LimitationToModify(
        limitationName="USE_MATERIAL_VIEWS",
        settingUpgradeFunction=lambda oldValue: not oldValue,
        settingDowngradeFunction=lambda oldValue: not oldValue if oldValue else oldValue,
    ),
]


# DO NOT CHANGE MANUALLY - 'upgrade' function required for migration
def upgrade():
    """Upgrade configs to new revision"""

    # add groups
    mh.addGroup("new group", "new group description")

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_FACES_DB_PING_MAX_COUNT_2",
        validationSchema={"type": "string"},
        defaultValue="0",
        description="max count of database ping tries (<= 0 without ping)",
        services=["luna-faces"],
        groupNames=["luna-platform", "luna-queue"],
    )

    # rename limitations
    mh.renameLimitation("LUNA_FACES_LOGGER", "FACE_LOGGER")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    magicFunction()


# DO NOT CHANGE MANUALLY - 'downgrade' function required for migration
def downgrade():
    """Downgrade configs to previous revision"""

    # remove groups
    mh.removeGroup("new group")

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_FACES_DB_PING_MAX_COUNT_2")

    # rename limitations and its settings, tags, etc
    mh.renameLimitation("FACE_LOGGER", "LUNA_FACES_LOGGER")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
