"""
Module realizes base class for python service settings
"""

from abc import ABC
from typing import List, Optional

from configs_core.base_service_setting import BaseServiceSettingsClass

from ..settings.all_settings import SETTINGS
from ..settings.classes import LoggerSetting, MetricsSettings, MonitoringSettings, ServiceHttpSettings


class BasePythonServiceSettingsClass(BaseServiceSettingsClass, ABC):  # pylint: disable=W0223
    """
    Base luna-service settings class
    Attributes:
        logger(LoggerSetting): logger settings
        activePlugins(List[str]): list active plugin modules
        monitoring(MonitoringSettings): monitoring settings
        httpSettings(ServiceHttpSettings): http service settings
    """

    settingsStore = SETTINGS

    def __init__(self):
        self.logger: Optional[LoggerSetting] = None
        self.activePlugins: Optional[List[str]] = None
        self.metrics: Optional[MetricsSettings] = None
        self.monitoring: Optional[MonitoringSettings] = None
        self.httpSettings: Optional[ServiceHttpSettings] = None
        super().__init__()

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        self.logger = LoggerSetting(self.serviceName, self[f"{self._upperServiceName}_LOGGER"])
        self.activePlugins: List[str] = self[f"{self._upperServiceName}_ACTIVE_PLUGINS"]
        self.metrics = MetricsSettings(self.serviceName, self["LUNA_SERVICE_METRICS"])
        self.monitoring = MonitoringSettings(self.serviceName, self["LUNA_MONITORING"])
        self.httpSettings = ServiceHttpSettings(self.serviceName, self[f"{self._upperServiceName}_HTTP_SETTINGS"])

    def _storeSettings(self, jsonConfig: dict):
        """
        Deserialize loaded settings to classes
        """
        super()._storeSettings(jsonConfig)
        self.applySettings()
