"""
Module contains settings class for luna-video-agent service
"""

from dataclasses import dataclass
from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    AgentSettings as _AgentSettings,
    AgentVideoSettings as _AgentVideoSettings,
    EstimatorRuntimeSetting as _EstimatorRuntimeSetting,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    RuntimeSetting as _RuntimeSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
    TERuntimeSetting as _TERuntimeSetting,
    TESettings as _TESettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


@dataclass
class EstimatorsSettings:
    """
    Estimators settings collection
    """

    faceWarpEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyWarpEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceLandmarks5Estimator: Optional[_EstimatorRuntimeSetting] = None
    headPoseEstimator: Optional[_EstimatorRuntimeSetting] = None
    humanDetector: Optional[_EstimatorRuntimeSetting] = None
    peopleCountEstimator: Optional[_EstimatorRuntimeSetting] = None
    deepfakeEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceWarpQualityEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceDescriptorEstimator: Optional[_EstimatorRuntimeSetting] = None
    basicAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyDescriptorEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    emotionsEstimator: Optional[_EstimatorRuntimeSetting] = None
    gazeEstimator: Optional[_EstimatorRuntimeSetting] = None
    eyesAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    livenessEstimator: Optional[_EstimatorRuntimeSetting] = None
    maskEstimator: Optional[_EstimatorRuntimeSetting] = None
    glassesEstimator: Optional[_EstimatorRuntimeSetting] = None
    mouthAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    agsEstimator: Optional[_EstimatorRuntimeSetting] = None
    trackEngine: Optional[_EstimatorRuntimeSetting] = None


class SettingsVideoAgent(BasePythonServiceSettingsClass):
    """
    Settings class for luna-video-agent
    """

    serviceName = "luna-video-agent"

    def __init__(self):
        super().__init__()
        self.videoSettings: Optional[_AgentVideoSettings] = None
        self.runtimeSettings: Optional[_RuntimeSetting] = None
        self.agentSettings: Optional[_AgentSettings] = None
        self.videoManagerAddress: Optional[_ServiceAddressSettings] = None
        self.videoManagerTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.licensesAddress: Optional[_ServiceAddressSettings] = None
        self.imageOriginStorage: Optional[_ImageStoreAddressSettings] = None
        self.imageOriginStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.lunaSenderAddress: Optional[_ServiceAddressSettings] = None
        self.lunaSenderTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.lunaEventsAddress: Optional[_ServiceAddressSettings] = None
        self.lunaEventsTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.objectsStorage: Optional[_ImageStoreAddressSettings] = None
        self.externalApiAddress: Optional[_ServiceAddressSettings] = None
        self.handlersAddress: Optional[_ServiceAddressSettings] = None
        self.handlersTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.trackEngineSettings: Optional[_TESettings] = None
        self.estimatorsSettings: Optional[EstimatorsSettings] = None
        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.defaultHumanDescriptorVersion: Optional[int] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.estimatorsSettings = EstimatorsSettings()
        self.videoSettings = _AgentVideoSettings(
            self.serviceName, "LUNA_VIDEO_AGENT_VIDEO_SETTINGS", self["LUNA_VIDEO_AGENT_VIDEO_SETTINGS"]
        )
        self.runtimeSettings = _RuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_RUNTIME_SETTINGS",
            self["LUNA_VIDEO_AGENT_RUNTIME_SETTINGS"],
        )
        self.agentSettings = _AgentSettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_SETTINGS",
            self["LUNA_VIDEO_AGENT_SETTINGS"],
        )
        self.videoManagerAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_VIDEO_MANAGER_ADDRESS", self["LUNA_VIDEO_MANAGER_ADDRESS"]
        )
        self.videoManagerTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_VIDEO_MANAGER_TIMEOUTS",
            self["LUNA_VIDEO_MANAGER_TIMEOUTS"],
        )
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )
        self.lunaSenderAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_SENDER_ADDRESS",
            self["LUNA_SENDER_ADDRESS"],
        )
        self.lunaSenderTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_SENDER_TIMEOUTS",
            self["LUNA_SENDER_TIMEOUTS"],
        )
        self.lunaEventsAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_EVENTS_ADDRESS",
            self["LUNA_EVENTS_ADDRESS"],
        )
        self.lunaEventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_EVENTS_TIMEOUTS",
            self["LUNA_EVENTS_TIMEOUTS"],
        )
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )
        self.objectsStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_OBJECTS_ADDRESS",
            self["LUNA_IMAGE_STORE_OBJECTS_ADDRESS"],
        )
        self.handlersAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_HANDLERS_ADDRESS", self["LUNA_HANDLERS_ADDRESS"]
        )
        self.handlersTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_HANDLERS_TIMEOUTS", self["LUNA_HANDLERS_TIMEOUTS"]
        )
        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.defaultHumanDescriptorVersion = self["DEFAULT_HUMAN_DESCRIPTOR_VERSION"]
        self.trackEngineSettings = _TESettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_HUMAN_TRACKER_SETTINGS",
            self["LUNA_VIDEO_AGENT_HUMAN_TRACKER_SETTINGS"]["estimator_settings"],
        )

        self.estimatorsSettings.livenessEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_LIVENESS_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_LIVENESS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.deepfakeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_DEEPFAKE_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_DEEPFAKE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.peopleCountEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_PEOPLE_COUNT_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_PEOPLE_COUNT_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.humanDetector = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_HUMAN_DETECTOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_HUMAN_DETECTOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.headPoseEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_HEAD_POSE_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_HEAD_POSE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceLandmarks5Estimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_FACE_LANDMARKS5_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_FACE_LANDMARKS5_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyWarpEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_BODY_WARP_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_BODY_WARP_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceWarpEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_FACE_WARP_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_FACE_WARP_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceWarpQualityEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_FACE_WARP_QUALITY_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_FACE_WARP_QUALITY_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceDescriptorEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.basicAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyDescriptorEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.emotionsEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_EMOTIONS_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_EMOTIONS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.gazeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_GAZE_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_GAZE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.eyesAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_EYES_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_EYES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.deepfakeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_DEEPFAKE_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_DEEPFAKE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.maskEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_MASK_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_MASK_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.glassesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_GLASSES_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_GLASSES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.mouthAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.agsEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_AGS_ESTIMATOR_SETTINGS",
            self["LUNA_VIDEO_AGENT_AGS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.trackEngine = _TERuntimeSetting(
            self.serviceName,
            "LUNA_VIDEO_AGENT_HUMAN_TRACKER_SETTINGS",
            self["LUNA_VIDEO_AGENT_HUMAN_TRACKER_SETTINGS"]["runtime_settings"],
        )
        return self
